unit DPRegstr;
{*********************************************************************}
{ Programmer:  Kevin S Gallagher     gallaghe@teleport.com            }
{                                                                     }
{ The code in this unit uses TRegIniFile for reading and writing      }
{ information from the registry. Since I want the code to work with   }
{ various versions of Delphi, the constants hold key information      }
{ except for the part which indicates the version of Delphi installed }
{ The function [GetDelphiInfo] is used to try and determine which     }
{ version of Delphi is currently installed. If there are more then    }
{ one version, the first version is assumed.                          }
{                                                                     }
{ The compiler directive KG_DEBUG is used only for enabling visual    }
{ feedback during debugging. If not defined then "Dialogs" unit isn't }
{ compiled into the code.                                             }
{                                                                     }
{ NOTE: The read and write routines/wrappers could be optimized. This }
{ job I have left to you. Why, so that you can learn from it. If you  }
{ get stuck then try again, if you are really stuck, send me a email  }
{ about the problem. Struggling is good for learning.                 }
{                                                                     }
{ NOTE: This entire unit is part of a project I created for managing  }
{ Delphi projects visually. It is located at                          }
{ http://www.delphifreestuff.com/                                     }
{ the current version at the above site only works up to D4, it can   }
{ easily be tweaked to work with higher versions. I have not worked   }
{ on it for sometime but still answer questions if there are any.     }
{ the unit (this unit) in the above site should be replaced with this }
{ one for starters.                                                   }
{*********************************************************************}

{$DEFINE KG_DEBUG}

interface

uses
  Windows, Messages, SysUtils, Classes,Controls, StdCtrls,FileCtrl,
  Registry {$IFDEF KG_DEBUG },Dialogs{$ENDIF};

{ Used with GetDelphiInfo right below }
type
  TDelphiInfo = Record
    ExeName:String ;
    Version:String ;
    InstallPath:String ;
  end ;


{ Used to determine the installed version of Delphi. Refer to
  the constants in this unit with depend on this information. }
function GetDelphiInfo(var recInfo: TDelphiInfo):boolean ;
function GetLocale:String ;

function DelphiLastSession:String;
function DelphiProjects:TStringList;
function GetSearchPath:String;
function GetProject(cName:String):String;
function SetSession(S:String):boolean;
function FlushProject:boolean;

function Len(S:String):LongInt;
function LastChrInStr(const S: String): Char;

procedure ReadCodeInSight;

function WhereIsDelphi:String;
function DelphiExe:String;

function SubStr(const S: string; C: Char):String;

procedure SetRegValue(_RootKey: HKEY;
                      _Localkey,
                      _Field: String;
                      V:Variant);

function GetRegValue(_RootKey: HKEY;
                     _Localkey,
                     _Field: String;
                     V:Variant):Variant;


const
  xUserKey     = HKEY_CURRENT_USER ;
  xMachince    = HKEY_LOCAL_MACHINE ;
  SessionKey   = 'Software\Borland\Delphi\%s\Session' ;
  SearchPath   = 'Software\Borland\Delphi\%s\Library' ;
  ProjectsKey  = 'Software\Borland\Delphi\%s\Closed Projects' ;
  xLocalDelphi = 'Software\Borland\Delphi\%s' ;
  xField       = 'Project' ;
  xDef         = 'ERROR' ;
  MAX_CODE     = 500 ;
  LF           = #10 ;
  CR           = #13 ;
  CRLF         : PChar = CR+LF ;

var
  aList:TStringList ;
  aLines:TStringList ;
  aString:array[0..MAX_CODE,0..2] of String ;

implementation


function GetDelphiInfo(var recInfo: TDelphiInfo):boolean ;
const
  LocalHome = 'Software\Borland\Delphi' ;
  FoundNoHome = 'Failed to find Delphi key in HKEY_LOCAL_MACHINE' ;
var
  Reg:      TRegistry ;
  aList:    TStringList ;
  cDelphi:  String ;
  iCount:   Integer ;
  i:        Integer ;
begin

  with RecInfo do
  begin
    ExeName := '?' ;
    Version := '?' ;
    InstallPath := '?' ;
  end ;

  Reg := TRegistry.Create ;
  try
    with Reg do
    begin
      RootKey := HKEY_LOCAL_MACHINE ;
      if OpenKey(LocalHome, False) then
      begin
        if HasSubKeys then
        begin
          aList := TStringList.Create ;
          try
            GetKeyNames(aList) ;
            iCount := aList.Count ;
            if iCount > -1 then
            begin
              { Attempt to get the first valid version of
                Delphi installed }
              for i := 0 to iCount -1 do
              begin
                CloseKey ;
                if OpenKey(LocalHome + '\' + aList.Strings[i], False) then
                  try
                    cDelphi := ReadString('Delphi ' + aList.Strings[i][1]) ;
                    CloseKey ;
                    if FileExists(cDelphi) then
                    begin
                      if OpenKey(LocalHome + '\' + aList.Strings[i], False) then
                        recInfo.InstallPath := ReadString('RootDir') ;
                      recInfo.ExeName := cDelphi ;
                      recInfo.Version := aList.Strings[i] ;
                      break ;
                    end ;
                  except
                    { Don't care to surface the exception to the UI }
                    CloseKey ;
                  end ;
              end ;
            end ;
          finally
            aList.Free ;
          end ;
        end ;
      end else
        Raise Exception.Create(FoundNoHome) ;
    end ;
  finally
    Reg.CloseKey ;
  end ;

  Result := True ;
end;

function GetDelphiVersion(FileName:String):String ;
var
  BufSize     : Integer ;
  InfoBuf     : PChar ;
  VerBufSize  : Integer ;
  VerBuf      : PChar ;
begin
  Result := '0.0' ;

  BufSize := GetFileVersionInfoSize(PChar(FileName),DWORD(BufSize)) ;
  if BufSize > 0 then
  begin
    InfoBuf := AllocMem(BufSize) ;
    try
      GetFileVersionInfo(PChar(FileName),0,BufSize,InfoBuf) ;

      { NOTE the use of function GetLocale (within this unit) }
      VerQueryValue(InfoBuf,
        PChar('StringFileInfo\' + GetLocale + '04E4\ProductVersion'),
        Pointer(VerBuf),DWORD(VerBufSize)) ;
        
      Result := VerBuf ;
    finally
      FreeMem(InfoBuf, BufSize) ;
    end ;
  end ;

end ;

procedure ReadCodeInSight;
  function RemCrLf(const S: String): String;
  var i:Integer;
  begin
    Result:=S;
    { Some strings of dual CRLF's }
    for i := 1 to 4 do
      if (LastChrInStr(Result)=LF) or (LastChrInStr(Result)=CR) then
        Delete(Result,Length(Result) -1,2);
  end;
var
  F:Textfile;
  cDir:String;
  S,cLine:String;
  cFile:String;
  i,x:Integer;
begin
  cFile :=  'Delphi32.dci';
  S := '';
  cDir := WhereIsDelphi + #92  + 'Bin';
  i := 0;

  if not DirectoryExists(cDir) then
    raise Exception('Could not locate ' + cDir) ;

  if not FileExists(cDir + #92 + cFile) then
    raise Exception('Could not locate ' + cFile) ;

  try
    AssignFile(F,cDir + #92 + cFile);
    Reset(F);

    ReadLn(F,cLine);

    repeat
      if Copy(cLine,1,1) = #91 then
      begin
        x := Pos('|',cLine);
        aString[i,0] := Copy(cLine,2,x -2 ) ;
        aString[i,1] := Copy(cLine,x + 2,Len(cLine)- (x +2));
        ReadLn(F,cLine);
      end else
      begin
        repeat
          aString[i,2] := aString[i,2] + cLine + CRLF;
          ReadLn(F,cLine);
          if eof(F) then
          begin
            aString[i,2] := aString[i,2] + cLine + CRLF ;
            break ;
          end ;
        until (Copy(cLine,1,1) = #91) ; // [
        aString[i,2] := RemCrLf(aString[i,2]) ;
        Inc(i) ;
      end;
    until (eof(F)) ;
    if len(cLine) > 0 then
      aString[i,2] := aString[i,2] + cLine ;
    aString[i,2] := RemCrLf(aString[i,2]) ;
  finally
    Closefile(F) ;
  end ;
end ;

function WhereIsDelphi:String;
const
  NoHome = 'Error getting Delphi home';
var
  recInfo: TDelphiInfo ;
  cKey: String ;
begin
  if GetDelphiInfo(recInfo) then
  begin
    cKey := Format(xLocalDelphi,[RecInfo.Version]) ;
    Result := GetRegValue(xMachince,cKey,'RootDir',NoHome) ;
  end ;
end;

function DelphiExe:String;
const
  xExeErr = 'Failed to get executable name';
begin
  Result := GetRegValue(xMachince,xLocalDelphi,'Delphi 4',xExeErr);
end;

function DelphiProjects:TStringList;
const
  xNotWantedChars = [#39,#44,#48..#57];
var
  S:String;
  Tmp:String;
  cName:String;
  x,i:Integer;
  iPos:Integer;
begin
  aList.Clear;
  for x := 0 to 4 do begin
    S:= GetProject('File_' + IntToStr(x));
    iPos := Pos(xDef,S);
    if iPos = 0 then begin
      Tmp := Copy(S,Pos(#39,S) ,Len(S) -  Pos(#39,S));
      cName := '';

      for i := 1 to Len(Tmp) do
        if Not(Tmp[i] in xNotWantedChars) then
          cName := cName + Tmp[i];
        aList.Add(cName);
    end;
  end;

  { You might wanted to check this yourself? }
  if aList.Count = 0 then
    raise Exception.Create('Failed to retreive any projects');

  Result := aList;
end;

function GetSearchPath:String;
var
  cKey: String ;
  recInfo: TDelphiInfo ;
  Reg: TRegistry ;
begin
  if GetDelphiInfo(recInfo) then
  begin
    cKey := Format(SearchPath,[RecInfo.Version]) ;
    Reg := TRegistry.Create ;
    try
      with Reg do
      begin
        RootKey := xUserKey ;
        if OpenKey(cKey, False) then
        begin
          if ValueExists('SearchPath') then
          begin
            Result := ReadString('SearchPath') ;
            CloseKey ;
          end ;
        end ;
      end ;
    finally
      Reg.Free ;
    end ;
  end ;
end;

function GetProject(cName:String):String;
var
  recInfo: TDelphiInfo ;
begin
  if GetDelphiInfo(recInfo) then
    Result := GetRegValue(xUserKey,Format(ProjectsKey,[RecInfo.Version]),cName,'');
end;

function SetSession(S:String):boolean;
var
  cComp:String;
  cKey: String ;
  recInfo: TDelphiInfo ;
begin
  Result := False ;

  if GetDelphiInfo(recInfo) then
  begin
    cKey := Format(SessionKey,[RecInfo.Version]) ;

    SetRegValue(xUserKey,cKey,xField,S);
    cComp := GetRegValue(xUserKey,cKey,xField,'');
    if cComp = S then Result := True;
  end ;

end;

function DelphiLastSession:String;
var
  recInfo: TDelphiInfo ;
begin
  if GetDelphiInfo(recInfo) then
    Result := GetRegValue(xUserKey,Format(SessionKey,[RecInfo.Version]),xField,'') ;
end;

function FlushProject:boolean;
var
  recInfo: TDelphiInfo ;
begin
  Result := False ;
  if GetDelphiInfo(recInfo) then
    SetRegValue(xUserKey,Format(SessionKey,[RecInfo.Version]),xField,'') ;

  if Len(DelphiLastSession) = 0 then  Result := True;
end;

procedure SetRegValue(_RootKey: HKEY;
                      _Localkey, _Field: String; V: Variant);
var
  TR: TRegIniFile;
begin
  TR := TRegIniFile.Create('');
  with TRegistry(TR) do begin
    RootKey := _RootKey;
    OpenKey(_Localkey,true);
    WriteString(_Field,V);
    CloseKey;
  end;
  TR.Free;
end;

function GetRegValue(_RootKey: HKEY;
                     _Localkey, _Field: String; V:Variant):Variant;
var
  TR: TRegIniFile;
begin
  Result := V;
  TR := TRegIniFile.Create('');
  with TRegistry(TR) do begin
    RootKey := _RootKey;
    if OpenKey(_Localkey,false) then begin
      if ValueExists(_Field) then
        Result := ReadString(_Field);
      CloseKey;
     end;
   end;
  TR.Free;
end;

{------------------------------------------------------------------------------
    Procedure:  GetLocale
  Description:  Used to obtain the country code for the current user
       Author:  Kevin S. Gallagher - gallaghe@teleport.com
 Date created:  02/20/2000
Date modified:
      Purpose:
 Known Issues:  None
 ------------------------------------------------------------------------------}
function GetLocale:String ;
var
  Reg: TRegistry ;
begin
  Reg := TRegistry.Create ;
  try
    with Reg do
    begin
      RootKey := HKEY_CURRENT_USER ;
      OpenKey('Control Panel\Desktop\ResourceLocale', False) ;
      Result := ReadString('') ;
      Result := Copy(Result,5,4) ;
      CloseKey ;
    end ;
  finally
    Reg.free ;
  end ;
end ;

function Len(S:String):LongInt;
begin
  Result := Length(S);
end;

function  LastChrInStr(const S: String): Char;
begin
  Result := S[Length(S)];
end;

function DelChars(const S: string; Chr: Char): string;
var
  I: Integer;
begin
  Result := S;
  for I := Len(Result) downto 1 do begin
    if Result[I] = Chr then Delete(Result, I, 1);
  end;
end;

function SubStr(const S: string; C: Char):String;
begin
  Result :=  Copy(S,Pos(C,S) +1 ,Len(S) -  Pos(C,S));
end;

initialization
  aList := TStringList.Create;
  aList.Duplicates := dupIgnore;
  aLines:= TStringList.Create;
finalization
  aList.Free;
  aLines.Free;

end.
