unit RestoreTreeNode;

{***************************************************************************}
{ Author                                                                    }
{   Kevin S. Gallagher                                                      }
{ Description                                                               }
{   Simple example for restoring the selected item in a TreeView. There are }
{   several things which must happen, get the selected item when closing the}
{   form and then save to the registry the path to the item. When opening   }
{   up the form we attempt get the path from the registry, then attempt to  }
{   make it the selected item again.                                        }
{ Comments                                                                  }
{   Remember this is a simple example, in real life a lot can go wrong i.e. }
{   the tree represents data from a table, the item last selected may be    }
{   deleted and not here not time we open the form...                       }
{ Required                                                                  }
{   A component called BreakApart, needs to be in the project directory or  }
{   within Delphi's lib/search path. It need not be installed.              }
{***************************************************************************}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, Brkapart, Registry;

type
  TfrmTreeNodeRestore = class(TForm)
    StatusBar1: TStatusBar;
    cmdClose: TButton;
    TreeView1: TTreeView;
    Label1: TLabel;
    procedure cmdCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

const
  LastNodeViewed = 'TreeNode' ;

var
  frmTreeNodeRestore: TfrmTreeNodeRestore;

implementation

uses MainForm;

{$R *.DFM}

procedure TfrmTreeNodeRestore.cmdCloseClick(Sender: TObject);
begin
  Close ;
end;

{--------------------------------------------------------
  Here we attempt to open a registry key which gets
  created when exiting this form and saves the last node
  you were on.
 -------------------------------------------------------}
procedure TfrmTreeNodeRestore.FormCreate(Sender: TObject);
  function Getit(oNode:TTreeNode; cCaption: String):TTreeNode ;
  begin
    { Make the compiler happy }
    Result := nil ;
    
    while oNode <> nil do
    begin
      if oNode.Text = cCaption then
      begin
        Result := oNode ;
        TreeView1.Selected := Result ;
        break ;
      end ;
      oNode := oNode.GetNext ;
    end ;
  end ;
var
  aNode:TTreeNode ;
  aTheOne:TTreeNode ;
  i,iCount:Integer ;
  BreakMe: TBrkApart ;
  Reg:TRegistry ;
  cKey:String ;
begin

  { You would load the tree here, or in the form which
    called it. }

  Reg := TRegistry.Create ;
  try
    { Open the key created when the main form was created. The
      constant "MyBaseKey" is located in the main form.
      Notice that OpenKeyReadOnly is used to open both keys.
      This makes an assumtion that they were created Okay in
      the OnClose event. }
    if Reg.OpenKeyReadOnly(MyBaseKey) then
    { This key is created in the OnClose event of this form,
      The constant MyTreeNodeKey is defined in the main form. }
      if Reg.OpenKeyReadOnly(MyTreeNodeKey) then
      begin
    { The value defined by "LastNodeViewed" which was created in
      the OnClose event holds the path of the TreeView when lasted
      viewed. The constant "LastNodeViewed" is defined in this
      unit. }
        cKey := Trim(Reg.ReadString(LastNodeViewed)) ;
        Reg.CloseKey ;

        if Length(cKey) = 0 then exit ;

        { This component will break apart the string read from the
          registry which is delimited by commas and place each
          section into a TStringList. This will be used to open the
          TreeView back up to the last place you were when exiting
          the form, hopefully. }
        BreakMe:= TBrkApart.Create(Self) ;
        try
          with BreakMe do
          begin
            BaseString := cKey ;
            BreakString := '\' ;
            BreakApart ;
            iCount := StringList.Count ;
          end ;

          if iCount = -1 then exit ;
          aNode := TreeView1.Items.Item[0] ;
          
          for i := 0 to iCount -1 do
          begin
            { Here I attempt to traverse the tree, opening branches
              as they are found. If a node is not located then we
              simply stop and leave the last found node the currently
              selected node. }
            aTheOne := GetIt(aNode,BreakMe.StringList.Strings[i]) ;
            if aTheOne <> nil then
            begin
              TreeView1.Selected := aTheOne;
              aNode := TreeView1.Selected ;
            end else
              break ;
          end ;
          ActiveControl := TreeView1 ;
        finally
          BreakMe.Free ;
        end ;

      end ;
  finally
    Reg.Free ;
  end ;

  ActiveControl := TreeView1 ;

end;

{--------------------------------------------------------
  This code saves the current node path as text to the
  registry. The OnCreate event will then attempt to open
  the node next time the form is displayed.
 -------------------------------------------------------}
procedure TfrmTreeNodeRestore.FormClose(Sender: TObject;
  var Action: TCloseAction);

  function  LastChrInStr(const S: String): Char;
  begin
    Result := S[Length(S)];
  end;

var
  Reg: TRegistry ;
  aNode: TTreeNode ;
  aList: TStringList ;
  i,iCount: Integer ;
  S:String ;
begin
  aNode := TreeView1.Selected ;

  if aNode <> nil then
  begin
    Reg := TRegistry.Create ;
    try
      { Open the key if it exist, otherwise it will be
        created since we passed True as the second parameter
        to OpenKey. The same is true for the sub-key below. }
      if Reg.OpenKey(MyBaseKey,True) then
        if Reg.OpenKey(MyTreeNodeKey,True) then
        begin

          { Traverse up the current node and place each node's
            text into a StringList. This StringList will then
            be used to build a string which represents the
            path to later open up to the same node in the OnCreate
            event of this form. }
          aList := TStringList.Create ;
          try
            while aNode <> nil do
            begin
              aList.Add(aNode.text) ;
              aNode := aNode.Parent ;
            end ;

            iCount := aList.Count ;

            if iCount > -1 then
            begin
              for i := iCount -1 downto 0 do
                S := S + aList.Strings[i] + '\';
              if LastChrInStr(S) = '\' then
                Delete(S,Length(S),1) ;

              { Now we write the path to the registry }
              Reg.WriteString(LastNodeViewed,S) ;
              Reg.CloseKey ;
            end ;
          finally
            aList.Free ;
          end ;
        end ;

    finally
      Reg.Free ;
    end ;
  end ;

  Action := caFree ;

end;

end.
